#!/usr/bin/perl

use strict;
use warnings;

if this is not a syntax error then I must be smoking something silly.

Just in case, lets die("so that it never ever ever runs till the end");

and return a false value in the end:

return undef;

__END__
